declare function isEmail(str: string): boolean;

declare function replaceAt(str: string, index?: number, replaceStr?: string): string;

declare function insertAt(str: string, index?: number, insertStr?: string): string;

declare function capitalize(str: string): string;

declare function camelCase(str: string): string;

declare function pascalCase(str: string): string;

declare function arrayDiff(arr1: unknown[], arr2: unknown[]): unknown[];

/**
 * It generates a sequence of numbers starting at the first argument,
 * progressing by the third argument, and stopping at the second argument.
 *
 * @example
 *
 * range(0, 5) // [0, 1, 2, 3, 4, 5]
 * range(10, 50, 10) // [10, 20, 30, 40, 50]
 */
declare function range(start: number | undefined, stop: number, step?: number): number[];

/**
 *  Array filter with `Async` callback function.
 *
 * @param arr unknown[]
 * @param cb - The callback funtion
 * @returns The filtered array
 */
declare function asyncFilter(arr: unknown[], cb: (value: unknown, index: number) => Promise<boolean>): Promise<unknown[]>;

type GroupKey<T> = ((a: T) => string) | string;
declare function groupBy<T>(arr: T[], key: GroupKey<T>): Record<string, T[]>;

/**
 * It moves an array element from one index position to another.
 *
 * Note: It mutates the array.
 *
 * @example
 *
 * move([1, 2, 3], 0, 2) //=> [2, 3, 1]
 */
declare function move<T>(arr: T[], from: number, to: number): T[];

type OrderType = 'asc' | 'desc';
type SortCB<T> = (a: T, b: T) => [unknown, unknown];
/**
 * Sorts a list of items and returns a new array.
 *
 * @example
 *
 * sort([1, 3, 2]) // [1, 2, 3]
 * sort(['x', 'z', 'y'], 'desc') // ['z', 'y', 'x']
 */
declare function sort<T>(arr?: T[], order?: OrderType, cb?: SortCB<T>): T[];

type OrderTuples<T> = [keyof T, OrderType][];
/**
 * Sorts a list of objects and returns a new array.
 *
 * @example
 *
 * const arr = [{a: 1}, {a: 3}, {a: 2}]
 * sortObj(arr, ['a', 'asc']); //=> [{a: 1}, {a: 2}, {a: 3}]
 */
declare function sortObj<T>(arr: T[], ...orderTyples: OrderTuples<T>): T[];

/**
 * It returns percentage of value & total.
 * @param {number} value - The current value
 * @param {number} total - The total value
 * @returns {number}
 */
declare function percentage(value: number, total: number): number;

/**
 * It returns percentage of a number.
 * @param {number} percentage - The percentage to calculate
 * @param {number} num - The number to find percentage
 * @returns {number}
 */
declare function percentageOf(percentage: number, num: number): number;

/**
 * Returns a tuple with Quotient and Remainder.
 *
 * @example
 *
 * divMod(4, 2) //=> [2, 0]
 * divMod(11, 4) //=> [2.75, 3]
 * divMod(1, 0) //=> [Infinity, NaN]
 */
declare function divMod(a: number, b: number): [number, number];

/**
 * Checks if the given number is an even number.
 *
 * @example
 *
 * even(1) //=> false
 * even(2) //=> true
 */
declare function even(n: number): boolean;

/**
 * Checks if the given number is an odd number.
 *
 * @example
 *
 * even(1) //=> true
 * even(2) //=> false
 */
declare function odd(n: number): boolean;

/**
 * Calculates the sum of values in the given array.
 *
 * @example
 *
 * sum([1, 2, 3, 4, 5]) //=> 15
 * sum([-1, -2]) //=> -3
 */
declare function sum(arr?: number[], cb?: (val: number, index: number) => number): number;

/**
 * Calculates the product of values in the given array.
 *
 * @example
 *
 * prod([1, 2, 3, 4, 5]) //=> 120
 * prod([-1, -2]) //=> 2
 */
declare function prod(arr?: number[], cb?: (val: number, index: number) => number): number;

/**
 * Calculates the mean value of the given array.
 *
 * @example
 *
 * mean([4, 1, 7]) //=> 4
 */
declare function mean(arr?: number[], cb?: (v: number, index: number) => number): number;

/**
 * Calculates the median value of the given array.
 *
 * @example
 *
 * median([1, 4, 2, 5, 0]) //=> 2
 *
 * median([10, 20, 40, 50]) //=> 30;
 */
declare function median(arr?: number[], cb?: (val: number, index: number) => number): number;

/**
 * Checks if the given value is number.
 *
 * @example
 *
 * isNum(1) //=> true
 *
 * isNum('1') //=> false
 *
 * isNum('1', true) //=> true
 *
 * isNum(NaN) //=> false
 *
 * isNum(Infinity) //=> false
 *
 */
declare function isNum(val: unknown, coerce?: boolean): val is number;

/**
 * Checks if the given value is a plain object.
 *
 * @example
 *
 * isObj({}) //=> true
 *
 * isObj([]) //=> false
 *
 * isObj(Object.create(null)) //=> true
 *
 */
declare function isObj(val: unknown): val is object;

/**
 * Checks whether the given string is a valid JSON plain object
 *
 * @example
 *
 * isJSON("null") //=> false
 *
 * isJSON("[]") //=> false
 *
 * isJSON("{}") //=> true
 */
declare function isJSON(str: string): boolean;

/**
 * Checks if the given value is boolean.
 *
 * @example
 *
 * isBool(true) //=> true
 *
 * isBool(false) //=> true
 *
 * isBool(null) //=> false
 */
declare function isBool(val: unknown): boolean;

/**
 * Checks if the given value is an array.
 *
 * @example
 *
 * isArr([]) //=> true
 *
 * isArr(new Array('a', 'b', 'c')) //=> true
 *
 * isArr('abc') //=> false
 */
declare function isArr(val: unknown): val is Array<unknown>;

/**
 * Checks if the given value is a valid date object.
 *
 * @example
 *
 * isDate(new Date()) //=> true
 *
 * isDate(new Date('')) //=> false
 *
 * isDate('2000-01-01') //=> false
 */
declare function isDate(val: unknown): val is Date;

/**
 * Checks if the given value is an error.
 *
 * @example
 *
 * isErr(new Error()) //=> true
 *
 * isErr({msg: '', name: ''}) //=> false
 */
declare function isErr(val: unknown): val is Error;

/**
 * Checks if the given value is a function.
 *
 * @example
 *
 * isFn(function() {}) //=> true
 *
 * isFn({}) //=> false
 */
declare function isFn(val: unknown): val is Function;

/**
 * Checks if the given value is an async function.
 *
 * @example
 *
 * isAsyFn(function() {}) //=> false
 *
 * isAsyFn(async function() {}) //=> true
 */
declare function isAsyFn(val: unknown): val is Function | AsyncGeneratorFunction;

/**
 * Checks if the given value is a generator function.
 *
 * @example
 *
 * isFn(function() {}) //=> false
 *
 * isFn(function*() {}) //=> true
 */
declare function isGenFn(val: unknown): val is GeneratorFunction;

/**
 * Checks if the given value is a Map object.
 *
 * @example
 *
 * isMap(new Map()) //=> true
 *
 * isMap(new WeakMap()) //=> false
 *
 * isMap({}) //=> false
 *
 */
declare function isMap(val: unknown): val is Map<unknown, unknown>;

/**
 * Checks if the given value is null.
 *
 * @example
 *
 * isNull(null) //=> true
 *
 * isNull(undefined) //=> false
 *
 */
declare function isNull(val: unknown): val is null;

/**
 * Checks if the given value is a RegExp object.
 *
 * @example
 *
 * isRegEx(new RegExp()) //=> true
 *
 * isRegEx(/a/) //=> true
 *
 * isRegEx({}) //=> false
 *
 */
declare function isRegEx(val: unknown): val is RegExp;

/**
 * Checks if the given value is a Set object.
 *
 * @example
 *
 * isSet(new Set()) //=> true
 *
 * isSet(new WeakSet()) //=> false
 *
 * isSet({}) //=> false
 *
 */
declare function isSet(val: unknown): val is Set<unknown>;

/**
 * Checks if the given value is a string.
 *
 * @example
 *
 * isStr('abc') //=> true
 *
 * isStr(String(1)) //=> true
 *
 * isStr(1) //=> false
 *
 */
declare function isStr(val: unknown): val is string;

/**
 * Checks if the given value is a Symbol.
 *
 * @example
 *
 * isSym(Symbol()) //=> true
 *
 * isSym(Symbol('abc')) //=> true
 *
 * isSym('abc') //=> false
 *
 */
declare function isSym(val: unknown): val is symbol;

/**
 * Checks if the given value is a TypedArray.
 *
 * @example
 *
 * isTypedArr([]) //=> false
 *
 * isTypedArr(new Int8Array()) //=> true
 *
 * isTypedArr(new Float64Array()) //=> true
 */
type TypedArray = Int8Array | Uint8Array | Uint8ClampedArray | Int16Array | Uint16Array | Int32Array | Uint32Array | Float32Array | Float64Array | BigInt64Array | BigUint64Array;
declare function isTypedArr(val: unknown): val is TypedArray;

/**
 * Checks if the given value is an ArrayBuffer.
 *
 * @example
 *
 * isArrBuf([]) //=> false
 *
 * isArrBuf(new ArrayBuffer(8)) //=> true
 */
declare function isArrBuf(val: unknown): val is ArrayBuffer;

/**
 * Checks if the given value is undefined.
 *
 * @example
 *
 * isUndef() //=> true
 *
 * isUndef(undefined) //=> true
 *
 * isUndef(null) //=> false
 *
 */
declare function isUndef(val: unknown): val is undefined;

/**
 * Checks if the given value is a WeakMap object.
 *
 * @example
 *
 * isWkMap(new Map()) //=> false
 *
 * isWkMap(new WeakMap()) //=> true
 *
 */
declare function isWkMap(val: unknown): val is WeakMap<WeakKey, unknown>;

/**
 * Checks if the given value is a WeakSet object.
 *
 * @example
 *
 * isWkSet(new Set()) //=> false
 *
 * isWkSet(new WeakSet()) //=> true
 *
 */
declare function isWkSet(val: unknown): val is WeakSet<WeakKey>;

/**
 * Checks if the given value is a WeakRef object.
 *
 * @example
 *
 * isWkRef({}) //=> false
 *
 * let user = { name: "John" };
 * isWkRef(new WeakRef(user)) //=> true
 *
 */
declare function isWkRef(val: unknown): val is WeakRef<WeakKey>;

/**
 * Checks if the given value is a BigInt.
 *
 * @example
 *
 * isBigInt(1) //=> false
 *
 * isBigInt(1n) //=> true
 *
 */
declare function isBigInt(val: unknown): val is bigint;

/**
 * Checks if the given value is a blob.
 *
 * @example
 *
 * isBlob(new Blob()) //=> true
 *
 * isBlob({}) //=> false
 */
declare function isBlob(val: unknown): val is Blob;

/**
 * Checks if the given value is a Promise object.
 *
 * @example
 *
 * isPromise({}) //=> false
 *
 * isPromise(Promise.resolve()) //=> true
 */
declare function isPromise(val: unknown): val is Promise<unknown>;

/**
 * Checks if the given value is an object and not null.
 *
 * @example
 *
 * isPureObj({}) //=> true
 *
 * isPureObj([]) //=> true
 *
 * isPureObj(null) //=> false
 *
 */
declare function isPureObj(val: unknown): val is object;

/**
 * Checks if the given value is defined.
 *
 * @example
 *
 * isDef() //=> false
 * isDef(undefined) //=> false
 * isDef(null) //=> true
 * isDef('') //=> true
 *
 */
declare function isDef(val: unknown): boolean;

declare function getInObj(obj: object, path: string): unknown;

declare function setInObj(obj: object, path: string, value: unknown): object;

declare function delInObj(obj: unknown, path: string): boolean;

declare function clone<T>(val: T): T;

/**
 * Checks if two values are the same by shallow comparison.
 *
 * @example
 *
 * const a = {a: 1}
 * const b = {a: 1}
 *
 * a === b //=> false
 *
 * isShallowEql(a, b) //=> true
 *
 * isShallowEql([1, b], [1, b]) //=> true
 *
 * isShallowEql([1, {a: 1}], [1, {a: 1}]) //=> false
 */
declare function isShallowEql(val1: unknown, val2: unknown): boolean;

/**
 * Convert hexadecimal color values into RGB color values.
 * @param hex string
 * @returns number[]
 *
 * @example hexToRGB('#FF851B') => [ 255, 133, 27 ]
 */
declare function hexToRGB6(hex: string): number[];

declare function sleep(ms: number): Promise<void>;

/**
 * Returns a value clamped to the inclusive range of min and max.
 *
 * @example
 *
 * clamp(0, 1000, 1366) //=> 1000
 * clamp(1000, 1000, 1366) //=> 1000
 * clamp(1001, 1000, 1366) //=> 1001
 * clamp(1500, 1000, 1366) //=> 1366
 */
declare function clamp(val: number, min: number, max: number): number;

/**
 * Checks if the given number is positive zero (0).
 *
 * @example
 *
 * isZero() //=> false
 * isZero(null) //=> false
 * isZero('') //=> false
 * isZero(-0) //=> false
 * isZero(0) //=> true
 */
declare function isZero(n: number): boolean;

/**
 * Checks if the given number is negative zero (-0).
 *
 * @example
 *
 * isNegZero() //=> false
 * isNegZero(null) //=> false
 * isNegZero('') //=> false
 * isNegZero(0) //=> false
 * isNegZero(-0) //=> true */
declare function isNegZero(n: number): boolean;

/**
 * Try to convert the given value into a finite number.
 *
 * @example
 *
 * toNum(1) //=> 1
 * toNum('1.3') //=> 1.3
 * toNum('1_000') //=> 1000
 * toNum(Infinity) //=> NaN
 */
declare function toNum(val: unknown): number;

export { arrayDiff, asyncFilter, camelCase, capitalize, clamp, clone, delInObj, divMod, even, getInObj, groupBy, hexToRGB6 as hexToRGB, insertAt, isArr, isArrBuf, isAsyFn, isBigInt, isBlob, isBool, isDate, isDef, isEmail, isErr, isFn, isGenFn, isJSON, isMap, isNegZero, isNull, isNum, isObj, isPromise, isPureObj, isRegEx, isSet, isShallowEql, isStr, isSym, isTypedArr, isUndef, isWkMap, isWkRef, isWkSet, isZero, mean, median, move, odd, pascalCase, percentage, percentageOf, prod, range, replaceAt, setInObj, sleep, sort, sortObj, sum, toNum };
